/* $Id$

This file is part of NetVis
Copyright @ 2008 Mikko Rasa, Mikkosoft Productions
Distributed unter the GPL
*/

#include <cstdlib>
#include <netinet/in.h>
#include <netdb.h>
#include <msp/gl/immediate.h>
#include <msp/gl/matrix.h>
#include <msp/gl/meshbuilder.h>
#include <msp/gl/texture.h>
#include <msp/strings/lexicalcast.h>
#include "port.h"
#include "netvis.h"

using namespace std;
using namespace Msp;

Port::Port(NetVis &v, unsigned n):
	netvis(v),
	number(n),
	mesh((GL::COLOR4_UBYTE, GL::VERTEX2))
{
	char buf[128];
	sockaddr_in addr;
	addr.sin_family = AF_INET;
	addr.sin_port = ntohs(number);
	addr.sin_addr.s_addr = 0;
	int err = getnameinfo(reinterpret_cast<sockaddr *>(&addr), sizeof(sockaddr_in), 0, 0, buf, sizeof(buf), 0);
	if(err==0)
		name = buf;
	else
		name = Msp::lexical_cast(number);

	const map<unsigned, Port *> &ports = netvis.get_ports();
	unsigned tries = 100;
	if(ports.size()>100 && number>=1024)
		tries = 10000/ports.size()+1;
	float best_score = 0;
	for(unsigned i=0; (i<tries && best_score<1); ++i)
	{
		GL::Color c;
		c.r = rand()*1.0/RAND_MAX;
		c.g = rand()*1.0/RAND_MAX;
		c.b = rand()*1.0/RAND_MAX;
		float high = max(max(c.r, c.g), c.b);
		c = c*(1.0/high);
		if(number<1024)
		{
			float low = min(min(c.r, c.g), c.b);
			c = (c+-low)*(1/(1-low));
		}
		else
			c = c*0.6+0.4;

		float score = 2;
		for(map<unsigned, Port *>::const_iterator j=ports.begin(); j!=ports.end(); ++j)
		{
			if(number<1024 && j->first>1024)
				break;
			const GL::Color &other = j->second->get_color();
			float dr = c.r-other.r;
			float dg = c.g-other.g;
			float db = c.b-other.b;
			score = min(score, dr*dr+dg*dg+db*db);
		}
		if(score>best_score)
		{
			best_score = score;
			color = c;
		}
	}
	color.a = 0.4f;

	GL::MeshBuilder bld(mesh);
	bld.begin(GL::QUADS);
	bld.color(color.r, color.g, color.b, color.a);
	for(unsigned x=0; x<=4; x+=2)
	{
		bld.vertex(x+0, 0);
		bld.vertex(x+10, 0);
		bld.vertex(x+10, 10);
		bld.vertex(x+0, 10);
	}
	bld.end();
}

void Port::render() const
{
	GL::PushMatrix push_;
	mesh.draw();
	GL::translate(16, 1, 0);
	GL::scale_uniform(10);
	GL::Immediate imm((GL::COLOR4_UBYTE, GL::TEXCOORD2, GL::VERTEX2));
	netvis.get_font().draw_string(name, imm);
	GL::Texture::unbind();
}
